package de.intevation.flys.exports.minfo;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;
import org.w3c.dom.Document;

import de.intevation.artifactdatabase.state.ArtifactAndFacet;
import de.intevation.artifactdatabase.state.Facet;
import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.access.SedimentLoadAccess;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.minfo.SedimentLoadResult;
import de.intevation.flys.exports.StyledSeriesBuilder;
import de.intevation.flys.exports.XYChartGenerator;
import de.intevation.flys.jfree.FLYSAnnotation;
import de.intevation.flys.jfree.StyledXYSeries;


public class SedimentLoadLSGenerator
extends XYChartGenerator
implements FacetTypes
{
    public enum YAXIS {
        L(0);

        protected int idx;

        private YAXIS(int c) {
            idx = c;
        }
    }
    /** The logger that is used in this generator. */
    private static Logger logger = Logger.getLogger(BedQualityGenerator.class);

    public static final String I18N_CHART_TITLE = "chart.sedimentload.ls.title";
    public static final String I18N_XAXIS_LABEL = "chart.sedimentload.ls.xaxis.label";
    public static final String I18N_YAXIS_LABEL_1 = "chart.sedimentload.ls.yaxis.label.tpera";
    public static final String I18N_YAXIS_LABEL_2 = "chart.sedimentload.ls.yaxis.label.m3pera";

    public static final String I18N_CHART_TITLE_DEFAULT = "Sedimentfracht";
    public static final String I18N_XAXIS_LABEL_DEFAULT = "Fluss-Km";
    public static final String I18N_YAXIS_LABEL_DEFAULT_1 = "[t/a]";
    public static final String I18N_YAXIS_LABEL_DEFAULT_2 = "[m\u00b3/a]";

    private FLYSArtifact artifact;

    @Override
    protected YAxisWalker getYAxisWalker() {
        return new YAxisWalker() {

            @Override
            public int length() {
                return YAXIS.values().length;
            }

            @Override
            public String getId(int idx) {
                YAXIS[] yaxes = YAXIS.values();
                return yaxes[idx].toString();
            }
        };
    }

    @Override
    public void doOut(ArtifactAndFacet bundle, Document attr, boolean visible) {
        String name = bundle.getFacetName();

        logger.debug("doOut: " + name);

        if (name == null) {
            logger.error("No facet name for doOut(). No output generated!");
            return;
        }

        Facet facet = bundle.getFacet();
        artifact = (FLYSArtifact)bundle.getArtifact();

        if (facet == null) {
            return;
        }

        if (name.equals(SEDIMENT_LOAD_COARSE)) {
            doSedimentLoadCoarseOut(
                (SedimentLoadResult) bundle.getData(context),
                bundle,
                attr,
                visible);
        }
        else if (name.equals(SEDIMENT_LOAD_SAND)) {
            doSedimentLoadSandOut(
                (SedimentLoadResult) bundle.getData(context),
                bundle,
                attr,
                visible);
        }
        else if (name.equals(SEDIMENT_LOAD_FINEMIDDLE)) {
            doSedimentLoadFineMiddleOut(
                (SedimentLoadResult) bundle.getData(context),
                bundle,
                attr,
                visible);
        }
        else if (name.equals(SEDIMENT_LOAD_SUSP_SAND)) {
            doSedimentLoadSuspSandOut(
                (SedimentLoadResult) bundle.getData(context),
                bundle,
                attr,
                visible);
        }
        else if (name.equals(SEDIMENT_LOAD_SUSP_SAND_BED)) {
            doSedimentLoadSuspSandBedOut(
                (SedimentLoadResult) bundle.getData(context),
                bundle,
                attr,
                visible);
        }
        else if (name.equals(SEDIMENT_LOAD_SUSP_SEDIMENT)) {
            doSedimentLoadSuspSedimentOut(
                (SedimentLoadResult) bundle.getData(context),
                bundle,
                attr,
                visible);
        }
        else if (name.equals(SEDIMENT_LOAD_TOTAL)) {
            doSedimentLoadTotalOut(
                (SedimentLoadResult) bundle.getData(context),
                bundle,
                attr,
                visible);
        }
        else if (name.equals(LONGITUDINAL_ANNOTATION)) {
            doAnnotations(
                (FLYSAnnotation) bundle.getData(context),
                 bundle,
                 attr,
                 visible);
        }
    }

    @Override
    protected String getDefaultChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }

    @Override
    protected String getDefaultXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }

    @Override
    protected String getDefaultYAxisLabel(int pos) {
        String label = "default";
        if (pos == YAXIS.L.idx) {
            SedimentLoadAccess access = new SedimentLoadAccess(artifact);
            if (access.getUnit().equals("m3_per_a")) {
                label = msg(I18N_YAXIS_LABEL_2, I18N_YAXIS_LABEL_DEFAULT_2);
            }
            else {
                label = msg(I18N_YAXIS_LABEL_1, I18N_YAXIS_LABEL_DEFAULT_1);
            }
        }
        return label;
    }

    protected void doSedimentLoadCoarseOut(SedimentLoadResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getCoarseData(), true);

        addAxisSeries(series, YAXIS.L.idx, visible);
    }

    protected void doSedimentLoadSandOut(SedimentLoadResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getSandData(), true);

        addAxisSeries(series, YAXIS.L.idx, visible);
    }

    protected void doSedimentLoadFineMiddleOut(SedimentLoadResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getFineMiddleData(), true);

        addAxisSeries(series, YAXIS.L.idx, visible);
    }

    protected void doSedimentLoadSuspSandOut(SedimentLoadResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getSuspSandData(), true);

        addAxisSeries(series, YAXIS.L.idx, visible);
    }

    protected void doSedimentLoadSuspSandBedOut(SedimentLoadResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getSuspSandBedData(), true);

        addAxisSeries(series, YAXIS.L.idx, visible);
    }

    protected void doSedimentLoadSuspSedimentOut(SedimentLoadResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getSuspSedimentData(), true);

        addAxisSeries(series, YAXIS.L.idx, visible);
    }

    protected void doSedimentLoadTotalOut(SedimentLoadResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getTotalData(), true);

        addAxisSeries(series, YAXIS.L.idx, visible);
    }

}
